/*
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDeveloper
 */

using System;
using NUnit.Framework;

namespace TopCoder.Rules.DependencyMap.View
{

    /// <summary>
    /// Unit test fixture for Dependency.
    ///
    /// Testing of Dependency focuses on its various manipulation methods.
    /// </summary>
    [TestFixture]
    public class DependencyUnitTests
    {

        /// <summary>
        /// The title to use.
        /// </summary>
        private string title = "title";

        /// <summary>
        /// The shapeOverride to use.
        /// </summary>
        private int shapeOverride = 3;

        /// <summary>
        /// The dependency instance to test.
        /// </summary>
        private Dependency dependency = null;

        /// <summary>
        /// A dependant to use.
        /// </summary>
        private Dependency dependant1 = new Dependency("a");

        /// <summary>
        /// Another dependant to use.
        /// </summary>
        private Dependency dependant2 = new Dependency("b");

        /// <summary>
        /// Create a new dependency for each test.
        /// </summary>
        [SetUp]
        protected void SetUp() {
            dependency = new Dependency(title, shapeOverride);
        }

        /// <summary>
        /// Create instance with title.
        /// </summary>
        public void Create_Title() {
            dependency = new Dependency(title);
            Assert.AreEqual(title, dependency.Title, "Title mismatches.");
        }

        /// <summary>
        /// Create instance with null title.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void Create_NullTitle() {
            new Dependency(null);
        }

        /// <summary>
        /// Create instance with empty title.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void Create_EmptyTitle() {
            new Dependency(" ");
        }

        /// <summary>
        /// Create instance with title and shapeOverride.
        /// </summary>
        [Test]
        public void Create_TitleShapeOverride()
        {
            dependency = new Dependency(title, shapeOverride);
            Assert.AreEqual(title, dependency.Title, "Title mismatches.");
            Assert.AreEqual(shapeOverride, dependency.ShapeOverride, "ShapeOverride mismatches.");
        }

        /// <summary>
        /// Create instance with null title.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void Create_NullTitle2() {
            new Dependency(null, shapeOverride);
        }

        /// <summary>
        /// Create instance with empty title.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void Create_EmptyTitle2() {
            new Dependency(" ", shapeOverride);
        }

        /// <summary>
        /// Create instance with invalid shapeOverride.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentOutOfRangeException))]
        public void Create_InvalidShapeOverride()
        {
            new Dependency(title, 0);
        }

        /// <summary>
        /// Test getter Title.
        /// </summary>
        [Test]
        public void GetterTitle()
        {
            Assert.AreEqual(title, dependency.Title, "Title mismatches.");
        }

        /// <summary>
        /// Test getter ShapeOverride.
        /// </summary>
        [Test]
        public void GetterShapeOverride()
        {
            Assert.AreEqual(shapeOverride, dependency.ShapeOverride, "ShapeOverride mismatches.");
        }

        /// <summary>
        /// Test AddDependant().
        /// </summary>
        [Test]
        public void AddDependant()
        {
            dependency.AddDependant(dependant1);
            Assert.AreEqual(1, dependency.Dependants.Count, "Incorrect size.");
            Assert.IsTrue(dependency.Dependants.Contains(dependant1), "Does not contain the dependant.");

            dependency.AddDependant(dependant2);
            Assert.AreEqual(2, dependency.Dependants.Count, "Incorrect size.");
            Assert.IsTrue(dependency.Dependants.Contains(dependant1), "Does not contain the dependant.");
            Assert.IsTrue(dependency.Dependants.Contains(dependant2), "Does not contain the dependant.");
        }

        /// <summary>
        /// Test AddDependant() with null dependant.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void AddDependant_NullDependant()
        {
            dependency.AddDependant(null);
        }

        /// <summary>
        /// Test RemoveDependant().
        /// </summary>
        [Test]
        public void RemoveDependant()
        {
            dependency.AddDependant(dependant1);
            dependency.AddDependant(dependant2);

            dependency.RemoveDependant(dependant1.Title);
            Assert.AreEqual(1, dependency.Dependants.Count, "Incorrect size.");
            Assert.IsTrue(dependency.Dependants.Contains(dependant2), "Does not contain the dependant.");

            dependency.RemoveDependant(dependant2.Title);
            Assert.AreEqual(0, dependency.Dependants.Count, "Incorrect size.");
        }

        /// <summary>
        /// Test RemoveDependant() with null dependant.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void RemoveDependant_NullDependant()
        {
            dependency.RemoveDependant(null);
        }

        /// <summary>
        /// Test RemoveDependant() with empty dependant.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void RemoveDependant_EmptyDependant()
        {
            dependency.RemoveDependant(" ");
        }

        /// <summary>
        /// Test getter Dependants.
        /// </summary>
        [Test]
        public void GetterDependants()
        {
            dependency.AddDependant(dependant1);
            dependency.AddDependant(dependant2);

            Assert.AreEqual(2, dependency.Dependants.Count, "Incorrect size.");
            Assert.IsTrue(dependency.Dependants.Contains(dependant1), "Does not contain the dependant.");
            Assert.IsTrue(dependency.Dependants.Contains(dependant2), "Does not contain the dependant.");
        }

        /// <summary>
        /// Test indexer getter.
        /// </summary>
        [Test]
        public void GetterIndexer()
        {
            dependency.AddDependant(dependant1);
            dependency.AddDependant(dependant2);

            Assert.AreEqual(dependant1, dependency[dependant1.Title], "Incorrect dependant returned.");
            Assert.AreEqual(dependant2, dependency[dependant2.Title], "Incorrect dependant returned.");
        }

        /// <summary>
        /// Test indexer getter with null title.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void GetterIndexer_NullTitle()
        {
            Dependency dependant = dependency[null];
        }

        /// <summary>
        /// Test indexer getter with empty title.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void GetterIndexer_EmptyTitle()
        {
            Dependency dependant = dependency[" "];
        }

        /// <summary>
        /// Test indexer setter.
        /// </summary>
        [Test]
        public void SetterIndexer()
        {
            dependency[dependant1.Title] = dependant1;
            dependency[dependant2.Title] = dependant2;

            Assert.AreEqual(dependant1, dependency[dependant1.Title], "Incorrect dependant returned.");
            Assert.AreEqual(dependant2, dependency[dependant2.Title], "Incorrect dependant returned.");
        }

        /// <summary>
        /// Test indexer setter with null title.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void SetterIndexer_NullTitle()
        {
            dependency[null] = dependant1;
        }

        /// <summary>
        /// Test indexer setter with empty title.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void SetterIndexer_EmptyTitle()
        {
            dependency[" "] = dependant1;
        }

        /// <summary>
        /// Test indexer setter with null dependant.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void SetterIndexer_NullDependant()
        {
            dependency["dependant"] = null;
        }

    }

}
